<?php

namespace iDorMobile;

include_once 'DbUtils.php';
include_once 'GuiUtils.php';
include_once 'CFPropertyList/CFPropertyList.php';
require_once(dirname(__FILE__) . "/../utils/config.php");

use CFPropertyList;

class ApiSupport
{
    public static function isAuthenticated() {
        session_start();
        if (isset($_SESSION['user_level'])) {
            if ($_SESSION['user_level'] >= 20) {
                return true;
            }
        }
        return false;
    }

    public static function getCfgBlocks() {
        if(!ApiSupport::isAuthenticated()) {
            return array (
                "name" => "Configuration",
                "id" => "cfgBlocks",
                "children" => array
                (
                    0 => array (
                        "name" => "Login expired",
                        "id" => "NotAuth",
                        "help" => "Please login",
                        "isReadOnly" => true,
                        "children" => false
                    )
                ),
                'msg' => '<div id="error-msg">' . GuiUtils::getSessionLocalizedString('dbLoadFail') . '</div>',
                'success'=>false
            );
        }

        return array (
            "name" => "Configuration",
            "id" => "cfgBlocks",
            "help" => GuiUtils::getSessionLocalizedString('startHint'),
            "children" => array
            (
                0 => array (
                    "name" => GuiUtils::getSessionLocalizedString('apartments'),
                    "id" => "apartments",
                    "help" => GuiUtils::getSessionLocalizedString('apartmentsHint'),
                    "isReadOnly" => true,
                    "children" => true
                ),
                1 => array (
                    "name" => GuiUtils::getSessionLocalizedString('sipDoors'),
                    "id" => "sipDoors",
                    "help" => GuiUtils::getSessionLocalizedString('sipDoorsHint'),
                    "isReadOnly" => true,
                    "children" => true
                ),
                2 => array (
                    "name" => GuiUtils::getSessionLocalizedString('camera'),
                    "id" => "camera",
                    "help" => GuiUtils::getSessionLocalizedString('cameraHint'),
                    "children" => true
                ),
                3 => array (
                    "name" => GuiUtils::getSessionLocalizedString('controls'),
                    "id" => "controls",
                    "help" => GuiUtils::getSessionLocalizedString('controlsStartHint'),
                    "children" => true
                ),
                4 => array (
                    "name" => GuiUtils::getSessionLocalizedString('door'),
                    "id" => "door",
                    "help" => GuiUtils::getSessionLocalizedString('doorHint'),
                    "children" => true
                ),
                5 => array (
                    "name" => GuiUtils::getSessionLocalizedString('sipDefault'),
                    "id" => "sipDefault",
                    "help" => GuiUtils::getSessionLocalizedString('sipDefaultHint'),
                    "children" => true
                ),
                6 => array (
                    "name" => GuiUtils::getSessionLocalizedString('cfgBundle'),
                    "id" => "cfgBundle",
                    "help" => GuiUtils::getSessionLocalizedString('cfgBundleHint'),
                    "children" => true
                )
            )
        );
    }

    public static function getApartments() {
        if(!ApiSupport::isAuthenticated()) {
            return array (
                "name" => "Apartments",
                "id" => "apartments",
                "children" => array
                (
                    0 => array (
                        "name" => "Login expired",
                        "id" => "NotAuth",
                        "help" => "Please login",
                        "isReadOnly" => true,
                        "children" => false
                    )
                ),
                'msg' => '<div id="error-msg">' . GuiUtils::getSessionLocalizedString('dbLoadFail') . '</div>',
                'success'=>false
            );
        }

        $sql = 'SELECT id, name, number FROM apartments';
        $dbData = DBUtils::getDbData('sip', 'apartments', $sql, 0, 0);
        if(!empty($dbData) && isset($dbData['error'])) {
            return $dbData['error'];
        }

        if(empty($dbData)) {
            return array(
                "name" => "Apartments",
                "id" => "apartments",
                "children"  => array (
                    0 => array(
                        "name" => GuiUtils::getSessionLocalizedString('apartmentsNotDefined'),
                        "id" => "0",
                        "help" => GuiUtils::getSessionLocalizedString('apartmentsNotDefinedHint'),
                        "parent" => "apartments",
                        "isReadOnly" => true,
                        "children" => false
                    )
                )
            );
        } else {
            $response = array(
                "name" => GuiUtils::getSessionLocalizedString('apartments'),
                "id" => "apartments",
                "children"  => array ());

            $i = 0;
            foreach($dbData as $row){
                $response["children"][$i] = array(
                    "name" => $row["name"],
                    "id" => 'sipAccounts/' . $row["id"],
                    "realId" => $row["id"],
                    "help" => sprintf(GuiUtils::getSessionLocalizedString('apartmentHintMsg'), $row["name"], $row["number"]),
                    "parent" => "apartments",
                    "hasDetails" => true,
                    "isReadOnly" => true,
                    "children" => true
                );
                $i++;
            }
            return $response;
        }
    }

    public static function getSipDoors() {
        if(!ApiSupport::isAuthenticated()) {
            return array (
                "name" => "Sip Doors",
                "id" => "sipDoors",
                "children" => array
                (
                    0 => array (
                        "name" => "Login expired",
                        "id" => "NotAuth",
                        "help" => "Please login",
                        "isReadOnly" => true,
                        "children" => false
                    )
                ),
                'msg' => '<div id="error-msg">' . GuiUtils::getSessionLocalizedString('dbLoadFail') . '</div>',
                'success'=>false
            );
        }

        $sql = 'SELECT id, name, location FROM door_modules';
        $dbData = DBUtils::getDbData('sip', 'doors', $sql, 0, 0);
        if(!empty($dbData) && isset($dbData['error'])) {
            return $dbData['error'];
        }

        if(empty($dbData)) {
            return array(
                "name" => "Sip Doors",
                "id" => "sipDoors",
                "children"  => array (
                    0 => array(
                        "name" => GuiUtils::getSessionLocalizedString('doorsNotDefined'),
                        "id" => "1",
                        "help" => GuiUtils::getSessionLocalizedString('doorsNotDefinedHint'),
                        "parent" => "sipDoors",
                        "isReadOnly" => true,
                        "children" => false
                    )
                )
            );
        } else {
            $response = array(
                "name" => GuiUtils::getSessionLocalizedString('sipDoors'),
                "id" => "sipDoors",
                "children"  => array ());

            $i = 0;
            foreach( $dbData as $row){
                $response["children"][$i] = array(
                    "name" => $row["name"],
                    "id" => $row["id"],
                    "help" => sprintf(GuiUtils::getSessionLocalizedString('doorHintMsg'), $row["name"], $row["location"]),
                    "parent" => "sipDoors",
                    "isReadOnly" => true,
                    "children" => false
                );
                $i++;
            }

            return $response;
        }
    }

    public static function getSipAccounts($apartmentId) {
        if(!ApiSupport::isAuthenticated()) {
            return array (
                "name" => "Sip Accounts",
                "id" => "sipAccounts",
                "children" => array
                (
                    0 => array (
                        "name" => "Login expired",
                        "id" => "NotAuth",
                        "help" => "Please login",
                        "isReadOnly" => true,
                        "children" => false
                    )
                ),
                'msg' => '<div id="error-msg">' . GuiUtils::getSessionLocalizedString('dbLoadFail') . '</div>',
                'success'=>false
            );
        }
        $sql = 'SELECT id, name, username FROM sip_accounts where apartment_id=';
        $dbData = DBUtils::getDbData('sip', 'sip accounts', $sql, $apartmentId, 0);
        if(!empty($dbData) && isset($dbData['error'])) {
            return $dbData['error'];
        }

        if(empty($dbData)) {
            return array(
                "name" => "Sip Accounts",
                "id" => "sipAccounts",
                "children"  => array (
                    0 => array(
                        "name" => GuiUtils::getSessionLocalizedString('sipAccountsNotDefined'),
                        "id" => "1",
                        "help" => GuiUtils::getSessionLocalizedString('sipAccountsNotDefinedHint'),
                        "parent" => "sipAccounts",
                        "isReadOnly" => true,
                        "children" => false
                    )
                )
            );
        } else {
            $response = array(
                "name" => GuiUtils::getSessionLocalizedString('sipAccounts'),
                "id" => "sipAccounts",
                "children"  => array ());

            $i = 0;
            foreach($dbData as $row){
                $response["children"][$i] = array(
                    "name" => $row["username"] . " - " . $row["name"],
                    "id" => $row["id"],
                    "help" => sprintf(GuiUtils::getSessionLocalizedString('sipAccountHint'), $row["name"], $row["username"]),
                    "parent" => "sipAccounts",
                    "isReadOnly" => true,
                    "children" => false
                );
                $i++;
            }

            return $response;
        }
    }

    public static function saveCamera() {
        if(!ApiSupport::isAuthenticated()) {
            return array (
                "name" => "Camera",
                "id" => "camera",
                "msg" => '<div id="error-msg">Login Expire. Please logout and login again.</div>',
                "success" => false
            );
        }

        $data = array();
        $data = json_decode(file_get_contents("php://input"), true);
        if(isset($data['camTypeId'])){
            $sql = 'SELECT name, standard_url FROM camera_types WHERE id=';
            $dbData = DBUtils::getDbData('cfg', 'cameras', $sql, $data['camTypeId'], 0);
            if(!empty($dbData) && isset($dbData['error'])) {
                return $dbData['error'];
            } else if (empty($dbData)){
                 array (
                    "name" => "Camera",
                    "id" => "camera",
                    "msg" => '<div id="error-msg">' . GuiUtils::getSessionLocalizedString('dbLoadFail') . '</div>',
                    'success'=>false);
            }

            $camUrl = isset($data['camUrl']) ? $data['camUrl'] : '';
            if($dbData[0]['name'] != 'Free Defined'){
                $camUrl = str_replace('host', isset($data['camHostname']) ? $data['camHostname'] : '', $dbData[0]['standard_url']);
                $camUrl = str_replace('port', isset($data['camPort']) ? $data['camPort'] : '80', $camUrl);
            }

            $sqlData = array('types' => 'sissssssi',
                'name' => $data['camName'],
                'camera_type_id' => $data['camTypeId'],
                'url' => $camUrl,
                'hostname' => isset($data['camHostname']) ? $data['camHostname'] : '',
                'port' => isset($data['camPort']) ? $data['camPort'] : '',
                'username' => isset($data['camUsername']) ? $data['camUsername'] : '',
                'password' => isset($data['camPassword']) ? $data['camPassword'] : '',
                'realm' => isset($data['camRealm']) ? $data['camRealm'] : '',
                'id' => isset($data['camId']) ? ($data['camId'] > 0 ? $data['camId'] : 0): 0);

            $query = "";

            $camId = 0;
            if(isset($data['camId']) && $data['camId'] > 0) {
                $camId = $data['camId'];
                $query = "UPDATE cameras SET name =?, camera_type_id =?, url =?, hostname =?, port =?, username =?, password =?, realm =? ";
                $query .= "WHERE id= ?";
            } else {
                $query = "INSERT INTO cameras (name, camera_type_id, url, hostname, port, username, password, realm, id) ";
                $query .= "VALUES (?,?,?,?,?,?,?,?,?)";
            }

            if(DBUtils::executeSql('cfg', 'camera', $query, $sqlData)) {
                if($camId == 0) {
                    $sql = "select id from cameras where name=";
                    $dbData = DBUtils::getDbData('cfg', 'cam configuration', $sql, $data['camName']);
                    if(isset($dbData['error'])) {
                        return array (
                            "name" => "Camera",
                            "id" => "camera",
                            'msg' => '<div id="error-msg">' . GuiUtils::getSessionLocalizedString('dbLoadFail') . '</div>',
                            'success'=>false);
                    }
                    $camId = $dbData[0]['id'];
                }

                $query = "delete from door_cam_link where cam_id=?";
                $sqlData = array('types' => 'i',
                    'id' => $camId);
                if(!DBUtils::executeSql('cfg', 'cam', $query, $sqlData, false)) {
                    return array (
                            "name" => "Camera",
                            "id" => "camera",
                            'msg' => '<div id="error-msg">' . GuiUtils::getSessionLocalizedString('dbLoadFail') . '</div>',
                            'success'=>false);
                }

                if(isset($data['camDoors'])){
                    foreach($data['camDoors'] as $doorId) {
                        $query = "insert into door_cam_link (door_id, cam_id) values (?,?)";
                        $sqlData = array('types' => 'ii',
                            'door_id' => $doorId,
                            'cam_id' => $camId);

                        if(!DBUtils::executeSql('cfg', 'door', $query, $sqlData)) {
                            return array (
                                    "name" => "Camera",
                                    "id" => "camera",
                                    'msg' => '<div id="error-msg">' . GuiUtils::getSessionLocalizedString('dbLoadFail') . '</div>',
                                    'success'=>false);
                        }
                    }
                }

                return array (
                    "name" => "Camera",
                    "id" => "camera",
                    'msg' => '<div id="load-ok-msg">' . GuiUtils::getSessionLocalizedString('dbLoadOk') . '</div>',
                    'success'=>true);
            }
        }

        return array (
            "name" => "Camera",
            "id" => "camera",
            'msg' => '<div id="error-msg">' . GuiUtils::getSessionLocalizedString('dbLoadFail') . '</div>',
            'success'=>false);;
    }

    public static function getCameras() {
        if(!ApiSupport::isAuthenticated()) {
            return array (
                "name" => "Cameras",
                "id" => "camera",
                "children" => array
                (
                    0 => array (
                        "name" => "Login expired",
                        "id" => "NotAuth",
                        "help" => "Please login",
                        "isReadOnly" => true,
                        "children" => false
                    )
                ),
                'msg' => '<div id="error-msg">' . GuiUtils::getSessionLocalizedString('dbLoadFail') . '</div>',
                'success'=>false
            );
        }
        $sql = 'SELECT id, name, url FROM cameras';
        $dbData = DBUtils::getDbData('cfg', 'cameras', $sql, 0, 0);
        if(!empty($dbData) && isset($dbData['error'])) {
            return $dbData['error'];
        }

        $response = array(
            "name" => "Cameras",
            "id" => "camera",
            "children"  => array (
                0 => array(
                    "name" => GuiUtils::getSessionLocalizedString('addNewCamera'),
                    "id" => "-1",
                    "help" => GuiUtils::getSessionLocalizedString('addCameraHint'),
                    "parent" => "camera",
                    "children" => false
        )));

        if(!empty($dbData)) {
            $i = 1;
            foreach( $dbData as $row){
                $response["children"][$i] = array(
                    "name" => $row["name"],
                    "id" => $row["id"],
                    "help" => GuiUtils::getSessionLocalizedString('addCameraHint'),
                    "parent" => "camera",
                    "children" => false
                );
                $i++;
            }
        }

        return $response;
    }

    public static function delCamera() {
        if(!ApiSupport::isAuthenticated()) {
            return array (
                "name" => "Camera",
                "id" => "camera",
                "msg" => '<div id="error-msg">Login Expire. Please logout and login again.</div>',
                "success" => false
            );
        }
        $data = array();
        $data = json_decode(file_get_contents("php://input"), true);

        $sqlData = array('types' => 'i', 'id' => $data['camId']);
        $query = "DELETE FROM cameras WHERE id= ?";

        if(DBUtils::executeSql('cfg', 'camera', $query, $sqlData)) {

            $query = "delete from door_cam_link where cam_id=?";
            $sqlData = array('types' => 'i',
                'id' => $data['camId']);
            if(!DBUtils::executeSql('cfg', 'cam', $query, $sqlData, false)) {
                return array (
                        "name" => "Camera",
                        "id" => "camera",
                        'msg' => '<div id="error-msg">' . GuiUtils::getSessionLocalizedString('dbDeleteFail') . '</div>',
                        'success'=>false);
            }

            return array (
                "name" => "Camera",
                "id" => "camera",
                'msg' => '<div id="load-ok-msg">' . GuiUtils::getSessionLocalizedString('dbDeleteOk') . '</div>',
                'success'=>true);
        }

        return array (
            "name" => "Camera",
            "id" => "camera",
            "data" => $data,
            'msg' => '<div id="error-msg">' . GuiUtils::getSessionLocalizedString('dbDeleteFail') . '</div>',
            'success'=>false);
    }

    public static function getControls() {
        if(!ApiSupport::isAuthenticated()) {
            return array (
                "name" => "Controls",
                "id" => "controls",
                "children" => array
                (
                    0 => array (
                        "name" => "Login expired",
                        "id" => "NotAuth",
                        "help" => "Please login",
                        "isReadOnly" => true,
                        "children" => false
                    )
                ),
                'msg' => '<div id="error-msg">' . GuiUtils::getSessionLocalizedString('dbLoadFail') . '</div>',
                'success'=>false
            );
        }
        return array (
            "name" => "Controls",
            "id" => "controls",
            "help" => GuiUtils::getSessionLocalizedString('controlsStartHint'),
            "children" => array
            (
                0 => array (
                    "name" => GuiUtils::getSessionLocalizedString('WR'),
                    "id" => "WR",
                    "help" => GuiUtils::getSessionLocalizedString('addWRHint'),
                    "isReadOnly" => false,
                    "children" => true
                ),
                1 => array (
                    "name" => GuiUtils::getSessionLocalizedString('DTMF'),
                    "id" => "DTMF",
                    "help" => GuiUtils::getSessionLocalizedString('addDTMFHint'),
                    "isReadOnly" => false,
                    "children" => true
                )
            )
        );
    }

    public static function getControlsByType($type) {
        if(!ApiSupport::isAuthenticated()) {
            return array (
                "name" => "Controls",
                "id" => $type,
                "children" => array
                (
                    0 => array (
                        "name" => "Login expired",
                        "id" => "NotAuth",
                        "help" => "Please login",
                        "isReadOnly" => true,
                        "children" => false
                    )
                ),
                'msg' => '<div id="error-msg">' . GuiUtils::getSessionLocalizedString('dbLoadFail') . '</div>',
                'success'=>false
            );
        }
        $sql = 'SELECT id, name FROM commands WHERE controller_type=';
        $dbData = DBUtils::getDbData('cfg', 'commands', $sql, $type, 0);
        if(!empty($dbData) && isset($dbData['error'])) {
            return $dbData['error'];
        }

        $response = array(
            "name" => "Controls",
            "id" => $type,
            "children"  => array (
                0 => array(
                    "name" => GuiUtils::getSessionLocalizedString('addNew' . $type),
                    "id" => "-1",
                    "help" => GuiUtils::getSessionLocalizedString('add' . $type . 'Hint'),
                    "parent" => $type,
                    "children" => false
        )));


        if(!empty($dbData)) {
            $i = 1;
            foreach( $dbData as $row){
                $response["children"][$i] = array(
                    "name" => $row["name"],
                    "id" => $row["id"],
                    "help" => GuiUtils::getSessionLocalizedString('add' . $type . 'Hint'),
                    "parent" => $type,
                    "children" => false
                );
                $i++;
            }
        }

        return $response;
    }

    public static function saveCommand() {
        if(!ApiSupport::isAuthenticated()) {
            return array (
                "name" => "Command",
                "id" => "command",
                "msg" => '<div id="error-msg">Login Expire. Please logout and login again.</div>',
                "success" => false
            );
        }
        $data = array();
        $data = json_decode(file_get_contents("php://input"), true);

        $sqlData = array('types' => 'ssssssssssssssii',
            'name' => $data['cmdName'],
            'controller_type' => isset($data['cmdCrtlType']) ? $data['cmdCrtlType'] : 'DTMF',
            'gui_type' => $data['cmdGuiType'],
            'dtmf_cmd_on' => isset($data['cmdDtmfOnTone']) ? $data['cmdDtmfOnTone'] : '',
            'on_title' => isset($data['cmdOnTitle']) ? $data['cmdOnTitle'] : '',
            'dtmf_cmd_off' => isset($data['cmdDtmfOffTone']) ? $data['cmdDtmfOffTone'] : '',
            'off_title' => isset($data['cmdOffTitle']) ? $data['cmdOffTitle'] : '',
            'dtmf_cmd' => isset($data['cmdDtmfTone']) ? $data['cmdDtmfTone'] : '',
            'color' => isset($data['cmdGuiColor']) ? $data['cmdGuiColor'] : '',
            'idm_relay_type' => isset($data['cmdRelayType']) ? $data['cmdRelayType'] : '',
            'idm_relay_host' => isset($data['cmdRelayHost']) ? $data['cmdRelayHost'] : '',
            'idm_relay_port' => isset($data['cmdRelayPort']) ? $data['cmdRelayPort'] : '80',
            'idm_relay_username' => isset($data['cmdRelayUser']) ? $data['cmdRelayUser'] : '',
            'idm_relay_password' => isset($data['cmdRelayPass']) ? $data['cmdRelayPass'] : '',
            'idm_relay_relay' => isset($data['cmdRelayRelay']) ? $data['cmdRelayRelay'] : '1',
            'id' => isset($data['cmdId']) ? ($data['cmdId'] > 0 ? $data['cmdId'] : 0): 0);

        $query = "";

        if(isset($data['cmdId']) && $data['cmdId'] > 0) {
            $query = "UPDATE commands SET name = ?, controller_type = ?,gui_type = ?, dtmf_cmd_on = ?, on_title = ?, dtmf_cmd_off = ?,";
            $query .= "off_title = ?, dtmf_cmd = ?, color = ?, idm_relay_type = ?, idm_relay_host = ?, ";
            $query .= "idm_relay_port = ?, idm_relay_username = ?, idm_relay_password = ?, idm_relay_relay = ? ";
            $query .= "WHERE id= ?";
        } else {
            $query = "INSERT INTO commands (name, controller_type, gui_type, dtmf_cmd_on, on_title, dtmf_cmd_off, off_title, dtmf_cmd, ";
            $query .= "color, idm_relay_type, idm_relay_host, idm_relay_port, idm_relay_username, idm_relay_password, idm_relay_relay, id) ";
            $query .= "VALUES (?,?,?,?, ?,?,?,? ,?,?,?,?, ?,?,?,?)";
        }

        if(DBUtils::executeSql('cfg', 'commnad', $query, $sqlData)) {
            return array (
                "name" => "Command",
                "id" => "commnad",
                "commandType" => isset($data['commandType']) ? $data['commandType'] : 'DTMF',
                'msg' => '<div id="load-ok-msg">' . GuiUtils::getSessionLocalizedString('dbLoadOk') . '</div>',
                'success'=>true);
        }

        return array (
            "name" => "Command",
            "id" => "commnad",
            'msg' => '<div id="error-msg">' . GuiUtils::getSessionLocalizedString('dbLoadFail') . '</div>',
            'success'=>false);;
    }

    public static function delCommand() {
        if(!ApiSupport::isAuthenticated()) {
            return array (
                "name" => "Command",
                "id" => "command",
                "msg" => '<div id="error-msg">Login Expire. Please logout and login again.</div>',
                "success" => false
            );
        }
        $data = array();
        $data = json_decode(file_get_contents("php://input"), true);

        $sqlData = array('types' => 'i', 'id' => $data['cmdId']);
        $query = "DELETE FROM commands WHERE id= ?";

        if(DBUtils::executeSql('cfg', 'camera', $query, $sqlData)) {
            return array (
                "name" => "Commands",
                "id" => "command",
                "commandType" => isset($data['commandType']) ? $data['commandType'] : 'DTMF',
                'msg' => '<div id="load-ok-msg">' . GuiUtils::getSessionLocalizedString('dbDeleteOk') . '</div>',
                'success'=>true);
        }

        return array (
            "name" => "Commands",
            "id" => "command",
            "data" => $data,
            'msg' => '<div id="error-msg">' . GuiUtils::getSessionLocalizedString('dbDeleteFail') . '</div>',
            'success'=>false);
    }

    public static function saveDoor() {
        if(!ApiSupport::isAuthenticated()) {
            return array (
                "name" => "Doors",
                "id" => "door",
                "msg" => '<div id="error-msg">Login Expire. Please logout and login again.</div>',
                "success" => false
            );
        }
        $data = array();
        $data = json_decode(file_get_contents("php://input"), true);
        $doorId = isset($data['doorId']) ? ($data['doorId'] > 0 ? $data['doorId'] : 0) : 0;
        $sqlData = array('types' => 'ssi',
            'name' => $data['doorName'],
            'msn' => $data['doorMsn'],
            'id' => $doorId);

        $query = "";

        if($doorId > 0) {
            $query = "UPDATE doors SET name =?, msn =? ";
            $query .= "WHERE id= ?";
        } else {
            $query = "INSERT INTO doors (name, msn, id) ";
            $query .= "VALUES (?,?,?)";
        }

        if(!DBUtils::executeSql('cfg', 'door', $query, $sqlData)) {
            return array (
            "name" => "Doors",
            "id" => "door",
            "data" => $data,
            'msg' => '<div id="error-msg">' . GuiUtils::getSessionLocalizedString('dbLoadFail') . '</div>',
            'success'=>false);
        }

        if($doorId == 0) {
            $sql = "select id from doors where name=";
            $dbData = DBUtils::getDbData('cfg', 'door configuration', $sql, $data['doorName']);
            if(isset($dbData['error'])) {
                return array (
                    "name" => "Doors",
                    "id" => "door",
                    'msg' => '<div id="error-msg">' . GuiUtils::getSessionLocalizedString('dbLoadFail') . '</div>',
                    'success'=>false);
            }
            $doorId = $dbData[0]['id'];
        }

        $query = "delete from door_cam_link where door_id=?";
        $sqlData = array('types' => 'i',
            'id' => $doorId);
        if(!DBUtils::executeSql('cfg', 'door', $query, $sqlData, false)) {
            return array (
                    "name" => "Doors",
                    "id" => "door",
                    'msg' => '<div id="error-msg">' . GuiUtils::getSessionLocalizedString('dbLoadFail') . '</div>',
                    'success'=>false);
        }

        if(isset($data['doorCams'])){
            foreach($data['doorCams'] as $camId) {
                $query = "insert into door_cam_link (door_id, cam_id) values (?,?)";
                $sqlData = array('types' => 'ii',
                    'door_id' => $doorId,
                    'cam_id' => $camId);

                if(!DBUtils::executeSql('cfg', 'door', $query, $sqlData)) {
                    return array (
                            "name" => "Doors",
                            "id" => "door",
                            'msg' => '<div id="error-msg">' . GuiUtils::getSessionLocalizedString('dbLoadFail') . '</div>',
                            'success'=>false);
                }
            }
        }

        return array (
                "name" => "Doors",
                "id" => "door",
                'msg' => '<div id="load-ok-msg">' . GuiUtils::getSessionLocalizedString('dbLoadOk') . '</div>',
                'success'=>true);
    }

    public static function getDoors() {
        if(!ApiSupport::isAuthenticated()) {
            return array (
                "name" => "Doors",
                "id" => "door",
                "children" => array
                (
                    0 => array (
                        "name" => "Login expired",
                        "id" => "NotAuth",
                        "help" => "Please login",
                        "isReadOnly" => true,
                        "children" => false
                    )
                ),
                'msg' => '<div id="error-msg">' . GuiUtils::getSessionLocalizedString('dbLoadFail') . '</div>',
                'success'=>false
            );
        }
        $sql = 'SELECT id, name FROM doors';
        $dbData = DBUtils::getDbData('cfg', 'door', $sql, 0, 0);
        if(!empty($dbData) && isset($dbData['error'])) {
            return $dbData['error'];
        }

        $response = array(
            "name" => "Doors",
            "id" => "door",
            "children"  => array (
                0 => array(
                    "name" => GuiUtils::getSessionLocalizedString('addNewDoor'),
                    "id" => "-1",
                    "help" => GuiUtils::getSessionLocalizedString('addNewDoorHint'),
                    "parent" => "door",
                    "children" => false
        )));

        if(!empty($dbData)) {
            $i = 1;
            foreach( $dbData as $row){
                $response["children"][$i] = array(
                    "name" => $row["name"],
                    "id" => $row["id"],
                    "help" => GuiUtils::getSessionLocalizedString('addNewDoorHint'),
                    "parent" => "door",
                    "children" => false
                );
                $i++;
            }
        }
        return $response;
    }

    public static function delDoor() {
        if(!ApiSupport::isAuthenticated()) {
            return array (
                "name" => "Doors",
                "id" => "door",
                "msg" => '<div id="error-msg">Login Expire. Please logout and login again.</div>',
                "success" => false
            );
        }
        $data = array();
        $data = json_decode(file_get_contents("php://input"), true);

        $query = "delete from door_cam_link where door_id=?";
        $sqlData = array('types' => 'i',
            'id' => $data['doorId']);
        if(!DBUtils::executeSql('cfg', 'door', $query, $sqlData, false)) {
            return array (
                    "name" => "Doors",
                    "id" => "door",
                    'msg' => '<div id="error-msg">' . GuiUtils::getSessionLocalizedString('dbDeleteFail') . '</div>',
                    'success'=>false);
        }

        $sqlData = array('types' => 'i', 'id' => $data['doorId']);
        $query = "DELETE FROM doors WHERE id= ?";

        if(DBUtils::executeSql('cfg', 'door', $query, $sqlData)) {
            return array (
                "name" => "Doors",
                "id" => "door",
                'msg' => '<div id="load-ok-msg">' . GuiUtils::getSessionLocalizedString('dbDeleteOk') . '</div>',
                'success'=>true);
        }

        return array (
            "name" => "Doors",
            "id" => "door",
            "data" => $data,
            'msg' => '<div id="error-msg">' . GuiUtils::getSessionLocalizedString('dbDeleteFail') . '</div>',
            'success'=>false);
    }

    public static function saveSipDefault() {
        if(!ApiSupport::isAuthenticated()) {
            return array (
                "name" => "Sip Defaults",
                "id" => "sipDefault",
                "msg" => '<div id="error-msg">Login Expire. Please logout and login again.</div>',
                "success" => false
            );
        }
        $data = array();
        $data = json_decode(file_get_contents("php://input"), true);

        $sqlData = array('types' => 'ssiissississsiiiiiiissssii',
            'name' => $data['sipDefName'],
            'SIPServerURL' => preg_replace('/\s+/', '', $data['sipDefSIPServerURL']),
            'SIPServerPort' => (isset($data['sipDefSIPServerPort']) && $data['sipDefSIPServerPort'] > 0) ? $data['sipDefSIPServerPort'] : 5060,
            'SIPAllowMakeCall' => isset($data['sipDefSIPAllowMakeCall']) ? $data['sipDefSIPAllowMakeCall'] : 0,

            'MainDoorMSN' => isset($data['sipDefMainDoorMSN']) ? $data['sipDefMainDoorMSN'] : "",
            'CallButtonName' => isset($data['sipDefCallButtonName']) ? $data['sipDefCallButtonName'] : "",
            'SIPNotificationBasedSystem' => isset($data['sipDefSIPNotificationBasedSystem']) ? $data['sipDefSIPNotificationBasedSystem'] : 0,
            'SIPDoorSystemType' => isset($data['sipDefSIPDoorSystemType']) ? $data['sipDefSIPDoorSystemType'] : "iDor",

            'ControlsPassword' => isset($data['sipDefControlsPassword']) ? $data['sipDefControlsPassword'] : "",
            'SIPUseOnlyUdp' => isset($data['sipDefSIPUseOnlyUdp']) ? $data['sipDefSIPUseOnlyUdp'] : 0,
            'OpenDoorDTMF' => isset($data['sipDefOpenDoorDTMF']) ? $data['sipDefOpenDoorDTMF'] : "",

            'AnswerDoorDTMF' => isset($data['sipDefAnswerDoorDTMF']) ? $data['sipDefAnswerDoorDTMF'] : "",
            'HangUpDTMF' => isset($data['sipDefHangUpDTMF']) ? $data['sipDefHangUpDTMF'] : "",
            'DelayDTMF' => isset($data['sipDefDelayDTMF']) ? $data['sipDefDelayDTMF'] : 0,
            'InbandDTMF' => isset($data['sipDefInbandDTMF']) ? $data['sipDefInbandDTMF'] : 0,

            'InbandDTMFClockRate' => isset($data['sipDefInbandDTMFClockRate']) ? $data['sipDefInbandDTMFClockRate'] : 8000,
            'InbandDTMFChannels' => isset($data['sipDefInbandDTMFChannels']) ? $data['sipDefInbandDTMFChannels'] : 1,
            'InbandDTMFSamplesPerFrame' => isset($data['sipDefInbandDTMFSamplesPerFrame']) ? $data['sipDefInbandDTMFSamplesPerFrame'] : 160,
            'InbandDTMFBitsPerSample' => isset($data['sipDefInbandDTMFBitsPerSample']) ? $data['sipDefInbandDTMFBitsPerSample'] : 16,
            'SIPUseStunServers' => isset($data['sipDefSIPUseStunServers']) ? $data['sipDefSIPUseStunServers'] : 0,

            'STUNServer1' => isset($data['sipDefSTUNServer1']) ? $data['sipDefSTUNServer1'] : "",
            'STUNServer2' => isset($data['sipDefSTUNServer2']) ? $data['sipDefSTUNServer2'] : "",
            'STUNServer3' => isset($data['sipDefSTUNServer3']) ? $data['sipDefSTUNServer3'] : "",
            'STUNServer4' => isset($data['sipDefSTUNServer4']) ? $data['sipDefSTUNServer4'] : "",
            'SIPDebugMode' => isset($data['sipDefSIPDebugMode']) ? $data['sipDefSIPDebugMode'] : 0,

            'id' => isset($data['sipDefId']) ? ($data['sipDefId'] > 0 ? $data['sipDefId'] : 0) : 0);

        $query = "";

        if(isset($data['sipDefId']) && $data['sipDefId'] > 0) {
            $query = "UPDATE sip_defaults SET name=?, SIPServerURL=?, SIPServerPort=?,SIPAllowMakeCall=?, ";
            $query .= "MainDoorMSN=?,CallButtonName=?, SIPNotificationBasedSystem=?,";
            $query .= "SIPDoorSystemType=?,ControlsPassword=?, SIPUseOnlyUdp=?, OpenDoorDTMF=?,AnswerDoorDTMF=?,HangUpDTMF=?,";
            $query .= "DelayDTMF=?, InbandDTMF=?,InbandDTMFClockRate=?,InbandDTMFChannels=?,InbandDTMFSamplesPerFrame=?,";
            $query .= "InbandDTMFBitsPerSample=?,SIPUseStunServers=?,STUNServer1=?,STUNServer2=?,STUNServer3=?,STUNServer4=?,SIPDebugMode=? ";
            $query .= "WHERE id= ?";
        } else {
            $query = "INSERT INTO sip_defaults (name, SIPServerURL, SIPServerPort,SIPAllowMakeCall,";
            $query .= "MainDoorMSN,CallButtonName, SIPNotificationBasedSystem, SIPDoorSystemType,ControlsPassword,";
            $query .= "SIPUseOnlyUdp, OpenDoorDTMF,AnswerDoorDTMF,HangUpDTMF,";
            $query .= "DelayDTMF, InbandDTMF,InbandDTMFClockRate,InbandDTMFChannels,InbandDTMFSamplesPerFrame,";
            $query .= "InbandDTMFBitsPerSample,SIPUseStunServers,STUNServer1,STUNServer2,STUNServer3,STUNServer4,SIPDebugMode, id) ";
            $query .= "VALUES (?,?,?,?, ?,?,?,?,?, ?,?,?,?, ?,?,?,?, ?,?,?,?,?, ?,?,?,?)";
        }

        if(DBUtils::executeSql('cfg', 'Sip Default Settings', $query, $sqlData)) {
            return array (
                "name" => "Sip Defaults",
                "id" => "sipDefault",
                'msg' => '<div id="load-ok-msg">' . GuiUtils::getSessionLocalizedString('dbLoadOk') . '</div>',
                'success'=>true);
        }

        return array (
            "name" => "Sip Defaults",
            "id" => "sipDefault",
            'msg' => '<div id="error-msg">' . GuiUtils::getSessionLocalizedString('dbLoadFail') . '</div>',
            'success'=>false);;
    }

    public static function getSipDefaults() {
        if(!ApiSupport::isAuthenticated()) {
            return array (
                "name" => "Sip Defaults",
                "id" => "sipDefault",
                "children" => array
                (
                    0 => array (
                        "name" => "Login expired",
                        "id" => "NotAuth",
                        "help" => "Please login",
                        "isReadOnly" => true,
                        "children" => false
                    )
                ),
                'msg' => '<div id="error-msg">' . GuiUtils::getSessionLocalizedString('dbLoadFail') . '</div>',
                'success'=>false
            );
        }
        $sql = 'SELECT id, name FROM sip_defaults';
        $dbData = DBUtils::getDbData('cfg', 'controller', $sql, 0, 0);
        if(!empty($dbData) && isset($dbData['error'])) {
            return $dbData['error'];
        }

         $response = array(
            "name" => "Sip Defaults",
            "id" => "sipDefault",
            "children"  => array (
                0 => array(
                    "name" => GuiUtils::getSessionLocalizedString('addNewSipDef'),
                    "id" => "-1",
                    "help" => GuiUtils::getSessionLocalizedString('addNewSipDefHint'),
                    "parent" => "sipDefault",
                    "children" => false
        )));

        if(!empty($dbData)) {
            $i = 1;
            foreach( $dbData as $row){
                $response["children"][$i] = array(
                    "name" => $row["name"],
                    "id" => $row["id"],
                    "help" => GuiUtils::getSessionLocalizedString('addNewSipDefHint'),
                    "parent" => "sipDefault",
                    "children" => false
                );
                $i++;
            }
        }
        return $response;
    }

    public static function delSipDefault() {
        if(!ApiSupport::isAuthenticated()) {
            return array (
                "name" => "Sip Defaults",
                "id" => "sipDefaults",
                "msg" => '<div id="error-msg">Login Expire. Please logout and login again.</div>',
                "success" => false
            );
        }
        $data = array();
        $data = json_decode(file_get_contents("php://input"), true);

        $sqlData = array('types' => 'i', 'id' => $data['sipDefId']);
        $query = "DELETE FROM sip_defaults WHERE id= ?";

        if(DBUtils::executeSql('cfg', 'controller', $query, $sqlData)) {
            return array (
                "name" => "Sip Defaults",
                "id" => "sipDefault",
                'msg' => '<div id="load-ok-msg">' . GuiUtils::getSessionLocalizedString('dbDeleteOk') . '</div>',
                'success'=>true);
        }

        return array (
            "name" => "Sip Defaults",
            "id" => "sipDefault",
            "data" => $data,
            'msg' => '<div id="error-msg">' . GuiUtils::getSessionLocalizedString('dbDeleteFail') . '</div>',
            'success'=>false);
    }

    public static function saveCfgBundle() {
        if(!ApiSupport::isAuthenticated()) {
            return array (
                "name" => "Config Bundles",
                "id" => "cfgBundle",
                "msg" => '<div id="error-msg">Login Expire. Please logout and login again.</div>',
                "success" => false
            );
        }
        $data = array();
        $data = json_decode(file_get_contents("php://input"), true);
        $cfgBundleId = isset($data['cfgBundleId']) ? ($data['cfgBundleId'] > 0 ? $data['cfgBundleId'] : 0) : 0;

        $oldCfgBundleId = -1;
        $sql = "SELECT id FROM config_bundles where level = 'Global'";
        $dbData = DBUtils::getDbData('cfg', 'config bundle', $sql, 0, 0);
        if(!empty($dbData) && !isset($dbData['error'])) {
            $oldCfgBundleId = $dbData[0]["id"];
        }

        if($data['cfgBundleLevel'] == 'Global' && $oldCfgBundleId > 0 && $oldCfgBundleId != $cfgBundleId) {
            return array (
                "name" => "Config Bundles",
                "id" => "cfgBundle",
                "msg" => '<div id="error-msg">Only one Global Config Bundle is allowed.</div>',
                "success" => false
            );
        }

        $sqlData = array('types' => 'ssii',
            'name' => $data['cfgBundleName'],
            'level' => $data['cfgBundleLevel'],
            'sip_defaults_id' => $data['cfgBundleSipDefId'],
            'id' => $cfgBundleId);

        $query = "";

        if($cfgBundleId > 0) {
            $query = "UPDATE config_bundles SET name =?, level =?, sip_defaults_id=? ";
            $query .= "WHERE id= ?";
        } else {
            $query = "INSERT INTO config_bundles (name, level, sip_defaults_id, id) ";
            $query .= "VALUES (?,?,?,?)";
        }

        if(!DBUtils::executeSql('cfg', 'door', $query, $sqlData)) {
            return array (
                "name" => "Config Bundles",
                "id" => "cfgBundle",
                "data" => $data,
                'msg' => '<div id="error-msg">' . GuiUtils::getSessionLocalizedString('dbLoadFail') . '</div>',
                'success'=>false);
        }

        if($cfgBundleId == 0) {
            $sql = "select id from config_bundles where name=";
            $dbData = DBUtils::getDbData('cfg', 'config bundle configuration', $sql, $data['cfgBundleName']);
            if(isset($dbData['error'])) {
                return array (
                    "name" => "Config Bundles",
                    "id" => "cfgBundle",
                    'msg' => '<div id="error-msg">' . GuiUtils::getSessionLocalizedString('dbLoadFail') . '</div>',
                    'success'=>false);
            }
            $cfgBundleId = $dbData[0]['id'];
        }

        $query = "delete from config_bundles_links where cfg_bundle_id=?";
        $sqlData = array('types' => 'i',
            'id' => $cfgBundleId);
        if(!DBUtils::executeSql('cfg', 'config bundles', $query, $sqlData, false)) {
            return array (
                    "name" => "Config Bundles",
                    "id" => "cfgBundle",
                    'msg' => '<div id="error-msg">' . GuiUtils::getSessionLocalizedString('dbLoadFail') . '</div>',
                    'success'=>false);
        }

        if(ApiSupport::saveCfgBundleAssoc($cfgBundleId, $data, "cfgBundleDoors", "Door") &&
            ApiSupport::saveCfgBundleAssoc($cfgBundleId, $data, "cfgBundleCams", "Camera") &&
            ApiSupport::saveCfgBundleAssoc($cfgBundleId, $data, "cfgBundleCmds", "Command")) {

            if($data['cfgBundleLevel'] == 'Apartment') {
                if(ApiSupport::saveCfgBundleAssoc($cfgBundleId, $data, "cfgBundleApartmentId", "Apartment")) {
                    return array (
                      "name" => "Config Bundles",
                      "id" => "cfgBundle",
                      'msg' => '<div id="load-ok-msg">' . GuiUtils::getSessionLocalizedString('dbLoadOk') . '</div>',
                      'success'=>true);
                }
            } else if($data['cfgBundleLevel'] == 'SipAccount') {
                if(ApiSupport::saveCfgBundleAssoc($cfgBundleId, $data, "cfgBundleUserId", "SipAccount")) {
                    return array (
                        "name" => "Config Bundles",
                        "id" => "cfgBundle",
                        'msg' => '<div id="load-ok-msg">' . GuiUtils::getSessionLocalizedString('dbLoadOk') . '</div>',
                        'success'=>true);
                }
            } else {
                return array (
                    "name" => "Config Bundles",
                    "id" => "cfgBundle",
                    'msg' => '<div id="load-ok-msg">' . GuiUtils::getSessionLocalizedString('dbLoadOk') . '</div>',
                    'success'=>true);
            }
        }

        return array (
            "name" => "Config Bundles",
            "id" => "cfgBundle",
            'msg' => '<div id="error-msg">' . GuiUtils::getSessionLocalizedString('dbLoadFail') . '</div>',
            'success'=>false);
    }

    private static function saveCfgBundleAssoc($cfgBundleId, $data, $dataName, $type) {
        if(isset($data[$dataName])){
            foreach($data[$dataName] as $assocId) {
                $query = "insert into config_bundles_links (type, cfg_bundle_id, assoc_id) values (?,?,?)";
                $sqlData = array('types' => 'sii',
                    'type' => $type,
                    'cfg_bundle_id' => $cfgBundleId,
                    'assoc_id' => $assocId);

                if(!DBUtils::executeSql('cfg', 'config bundle', $query, $sqlData)) {
                    return false;
                }
            }
        }
        return true;
    }

    public static function getCfgBundles() {
        if(!ApiSupport::isAuthenticated()) {
            return array (
                "name" => "Config Bundle",
                "id" => "cfgBundle",
                "children" => array
                (
                    0 => array (
                        "name" => "Login expired",
                        "id" => "NotAuth",
                        "help" => "Please login",
                        "isReadOnly" => true,
                        "children" => false
                    )
                ),
                'msg' => '<div id="error-msg">' . GuiUtils::getSessionLocalizedString('dbLoadFail') . '</div>',
                'success'=>false
            );
        }
        $sql = 'SELECT id, name FROM config_bundles';
        $dbData = DBUtils::getDbData('cfg', 'config bundle', $sql, 0, 0);
        if(!empty($dbData) && isset($dbData['error'])) {
            return $dbData['error'];
        }

        $response = array(
            "name" => "Config Bundles",
            "id" => "cfgBundle",
            "children"  => array (
                0 => array(
                    "name" => GuiUtils::getSessionLocalizedString('addNewCfgBundle'),
                    "id" => "-1",
                    "help" => GuiUtils::getSessionLocalizedString('addNewCfgBundleHint'),
                    "parent" => "cfgBundle",
                    "children" => false
        )));

        if(!empty($dbData)) {
            $i = 1;
            foreach( $dbData as $row){
                $response["children"][$i] = array(
                    "name" => $row["name"],
                    "id" => $row["id"],
                    "help" => GuiUtils::getSessionLocalizedString('addNewCfgBundleHint'),
                    "parent" => "cfgBundle",
                    "children" => false
                );
                $i++;
            }
        }
        return $response;
    }

    public static function delCfgBundle() {
        if(!ApiSupport::isAuthenticated()) {
            return array (
                "name" => "Config Bundles",
                "id" => "cfgBundle",
                "msg" => '<div id="error-msg">Login Expire. Please logout and login again.</div>',
                "success" => false
            );
        }
        $data = array();
        $data = json_decode(file_get_contents("php://input"), true);

        $query = "delete from config_bundles_links where cfg_bundle_id=?";
        $sqlData = array('types' => 'i',
            'id' => $data['cfgBundleId']);
        if(!DBUtils::executeSql('cfg', 'config bundle', $query, $sqlData, false)) {
            return array (
                    "name" => "Config Bundles",
                    "id" => "cfgBundle",
                    'msg' => '<div id="error-msg">' . GuiUtils::getSessionLocalizedString('dbDeleteFail') . '</div>',
                    'success'=>false);
        }

        $sqlData = array('types' => 'i', 'id' => $data['cfgBundleId']);
        $query = "DELETE FROM config_bundles WHERE id= ?";

        if(DBUtils::executeSql('cfg', 'config bundle', $query, $sqlData)) {
            return array (
                "name" => "Config Bundles",
                "id" => "cfgBundle",
                'msg' => '<div id="load-ok-msg">' . GuiUtils::getSessionLocalizedString('dbDeleteOk') . '</div>',
                'success'=>true);
        }

        return array (
            "name" => "Config Bundles",
            "id" => "cfgBundle",
            "data" => $data,
            'msg' => '<div id="error-msg">' . GuiUtils::getSessionLocalizedString('dbDeleteFail') . '</div>',
            'success'=>false);
    }

    public static function getiDorMobileSipServerUdid() {
        return exec("php /usr/iDorMobile/webgui/idorgui/deviceUdid.php");
    }

    public static function getConfig() {
        if(!isset($_GET['type']) || !isset($_GET['user']) || !isset($_GET['cfgKey'])){
            return array(
                "id" => "getConfig",
                "status" => 1,
                "data" => "error",
                "error" => "Missing information!");
        }

        $type = $_GET['type'];

        $sql = 'select id, apartment_id, username, password, authname from sip_accounts where username=';
        $dbData = DBUtils::getDbData('sip', 'get config', $sql, $_GET['user'], 1);
        if(empty($dbData) || isset($dbData['error'])) {
            return array(
                "id" => "getConfig",
                "status" => 2,
                "data" => "error",
                "error" => "No information for you!");
        }
        $localKey = "1D0rG3tUs3r:" . $dbData[0]['password'];
        $localKey = md5($localKey);
        $userPass = $dbData[0]['password'];
        $userAuthName = $dbData[0]['authname'];

        if($localKey != $_GET['cfgKey']) {
            return array(
                "id" => "getConfig",
                "status" => 2,
                "data" => "error",
                "error" => "No information for you!");
        }

        $sqlData = array('types' => 'i',
            'user_id' => $dbData[0]['id']
        );

        $query = "select cb.id, cb.level, cb.sip_defaults_id from config_bundles cb, config_bundles_links cbl ";
        $query .= "where cb.level='SipAccount' and cbl.type='SipAccount' and cbl.assoc_id=? and cb.id = cbl.cfg_bundle_id";

        $cfgBundleData = DBUtils::executeSql('cfg', 'getCfg', $query, $sqlData, true, true);
        if(empty($cfgBundleData)) {
            $sqlData = array('types' => 'i',
                'apartment_id' => isset($dbData[0]['apartment_id']) ? $dbData[0]['apartment_id'] : 0
            );
            $query = "select cb.id, cb.level, cb.sip_defaults_id from config_bundles cb, config_bundles_links cbl ";
            $query .= "where cb.level='Apartment' and cbl.type='Apartment' and cbl.assoc_id=? and cb.id = cbl.cfg_bundle_id";

            $cfgBundleData = DBUtils::executeSql('cfg', 'getCfg', $query, $sqlData, true, true);
            if(empty($cfgBundleData)) {
                $query = "select id, level, sip_defaults_id from config_bundles cb ";
                $query .= "where level=";
                $cfgBundleData = DBUtils::getDbData('cfg', 'getCfg', $query, 'Global', 1);

                if(empty($cfgBundleData) || isset($cfgBundleData['error'])) {
                    return array(
                        "id" => "getConfig",
                        "status" => 3,
                        "data" => "error",
                        "error" => "No information for you!");
                    }
                }
        }

        $query = "select * from sip_defaults where id=?";
        $sqlData = array('types' => 'i',
            'id' => $cfgBundleData[0]['sip_defaults_id']
        );

        $sipDefaultsData = DBUtils::executeSql('cfg', 'getCfg', $query, $sqlData, true, true);
        if(!isset($sipDefaultsData)) {
            return array(
                "id" => "getConfig",
                "status" => 4,
                "data" => "error",
                "error" => "No information for you!");
        }

        $sipDefaultsData[0]['SIPServerDeviceId'] = ApiSupport::getiDorMobileSipServerUdid();
        $sipDefaultsData[0]['SIPAuthName'] = $userAuthName;

        if(strcmp($sipDefaultsData[0]['SIPDoorSystemType'], 'iDM') == 0) {
            $sipDefaultsData[0]['SIPDoorSystemType'] = 0;
        }

        if(strcmp($sipDefaultsData[0]['SIPDoorSystemType'], 'Siedle') == 0) {
            $sipDefaultsData[0]['SIPDoorSystemType'] = 1;
        }

	if(strcmp($sipDefaultsData[0]['SIPDoorSystemType'], 'iDor') == 0) {
            $sipDefaultsData[0]['SIPDoorSystemType'] = 2;
        }

        $query = "select cam.id as CameraId, cam.name as CameraName, cam.url as CameraURL, cam.hostname as CameraHost, cam.port as CameraPort, ";
        $query .= "cam.username as CameraUsername, cam.password as CameraPassword, cam.realm as CameraRealm, ct.name as CameraType ";
        $query .= " from camera_types ct, cameras cam, config_bundles_links cbl where ";
        $query .= "cam.id=cbl.assoc_id and cbl.type=? and cbl.cfg_bundle_id=? and ct.id = cam.camera_type_id";
        $sqlData = array('types' => 'si',
            'type' => 'Camera',
            'id' => $cfgBundleData[0]['id']
        );

        $camsData = DBUtils::executeSql('cfg', 'getCfg', $query, $sqlData, false, true);
        if(!isset($sipDefaultsData)) {
            return array(
                "id" => "getConfig",
                "status" => 5,
                "data" => "error",
                "error" => "No information for you!");
        }

        $query = "select doors.id as DoorId, doors.name as DoorName, doors.msn as DoorMSN from doors, config_bundles_links cbl where doors.id=cbl.assoc_id and cbl.type=? and cbl.cfg_bundle_id=?";
        $sqlData = array('types' => 'si',
            'type' => 'Door',
            'id' => $cfgBundleData[0]['id']
        );

        $doorsData = DBUtils::executeSql('cfg', 'getCfg', $query, $sqlData, false, true);
        if(!isset($sipDefaultsData)) {
            return array(
                "id" => "getConfig",
                "status" => 6,
                "data" => "error",
                "error" => "No information for you!");
        }

        $doorsWithCamData = array();
        foreach($doorsData as $door) {
            $query = "select cam_id from door_cam_link where door_id=?";
            $sqlData = array('types' => 'i',
                'id' => $door['DoorId']
            );

            $doorCamsData = DBUtils::executeSql('cfg', 'getCfg', $query, $sqlData, false, true);
            if(isset($doorCamsData)) {
                $doorCams = array ();
                foreach($doorCamsData as $doorCam) {
                    $doorCams[] = $doorCam['cam_id'];
                }
                $door["DoorCameras"] = $doorCams;
            }

            $doorsWithCamData[] = $door;
        }


        $query = "select cmd.name as CmdText, cmd.controller_type as CmdCtrlType, ";
        $query .= "(CASE cmd.gui_type WHEN 'switch' THEN true ELSE false END) as IsSwitch, ";
        $query .= "cmd.dtmf_cmd_on as DTMFCmdOn, lower(cmd.on_title) as ImgCmdOn, cmd.dtmf_cmd_off as DTMFCmdOff, ";
        $query .= "lower(cmd.off_title) as ImgCmdOff, cmd.dtmf_cmd as DTMFCmd, cmd.idm_relay_type as CmdWrType, ";
        $query .= "lower(cmd.color) as CmdColor, ";
        $query .= "cmd.idm_relay_host as WrHost, cmd.idm_relay_port as WrPort, cmd.idm_relay_username as WrUser, ";
        $query .= "cmd.idm_relay_password as WrPass, cmd.idm_relay_relay as WrRelay";
        $query .= " from commands cmd, config_bundles_links cbl where cmd.id=cbl.assoc_id and cbl.type=? and cbl.cfg_bundle_id=?";
        $sqlData = array('types' => 'si',
            'type' => 'Command',
            'id' => $cfgBundleData[0]['id']
        );

        $cmdsData = DBUtils::executeSql('cfg', 'getCfg', $query, $sqlData, false, true);
        if(!isset($sipDefaultsData)) {
            return array(
                "id" => "getConfig",
                "status" => 6,
                "data" => "error",
                "error" => "No information for you!");
        }

        $configDate = date("Y-m-d H:i:s");
        $configBin = "";
        if($_GET['type'] == "ios") {
            $plist = new CFPropertyList\CFPropertyList();
            $plist->add( $dict = new CFPropertyList\CFDictionary() );
            $td = new CFPropertyList\CFTypeDetector();
            $camsStructure = $td->toCFType($camsData);
            $dict->add( 'CameraSettings', $camsStructure);
            $cmdsStructure = $td->toCFType($cmdsData);
            $dict->add( 'CmdSettings', $cmdsStructure);
            $doorsStructure = $td->toCFType($doorsWithCamData);
            $dict->add( 'DoorSettings', $doorsStructure);
            $sipDefStructure = $td->toCFType($sipDefaultsData[0]);
            $dict->add('SipSettings', $sipDefStructure);

            $configBin = $plist->toXML();

        } else {
            $configBin = array("CameraSettings" => $camsData,
                "CmdSettings" => $cmdsData,
                "DoorSettings" => $doorsWithCamData,
                "SipSettings" => $sipDefaultsData[0]);
            $configBin = json_encode($configBin);
        }

		// $configRaw = $configBin;
        $iv_size = mcrypt_get_iv_size(MCRYPT_DES, MCRYPT_MODE_CBC);
        $iv =  mcrypt_create_iv($iv_size, MCRYPT_RAND);
        $secretHash = "1D0rR3tUs3r:" . $userPass . $configDate;
        $secretHash = md5($secretHash);
        $configBin = openssl_encrypt($configBin, 'DES-CBC', $secretHash, 0, $iv);

        return array(
            "id" => "getConfig",
            //"raw" => $configRaw,
            "configDate" => $configDate,
            "configIV" => base64_encode($iv),
            "configBin" => $configBin);
    }

    public static function duplicateRecord()
    {
        if(!ApiSupport::isAuthenticated()) {
            return array (
                "name" => "Duplicate",
                "id" => "duplicate",
                "msg" => '<div id="error-msg">Login Expire. Please logout and login again.</div>',
                "success" => false
            );
        }
        $data = array();
        $data = json_decode(file_get_contents("php://input"), true);
        $origId = isset($data['dupId']) ? $data['dupId'] : 0;

        if(!isset($data['dupliType']) || $origId <= 0) {
            return array (
                "name" => "Duplicate",
                "id" => "duplicate",
                "data" => $data,
                "msg" => "<div id=\"error-msg\">" . GuiUtils::getSessionLocalizedString('dbLoadFail') . "</div>",
                "success"=>false);
        }


        switch($data['dupliType']) {
            case "command":
                $sqlData = array('types' => 'i',
                    'id' => $origId);

                $query = "INSERT INTO commands (name, controller_type, gui_type, dtmf_cmd_on, on_title, dtmf_cmd_off, off_title, dtmf_cmd, ";
                $query .= "color, idm_relay_type, idm_relay_host, idm_relay_port, idm_relay_username, idm_relay_password, idm_relay_relay) ";
                $query .= "SELECT CONCAT(name, ' Copy " . date("Y-m-d H:i:s") . "') as name, controller_type, gui_type, dtmf_cmd_on, on_title, dtmf_cmd_off, off_title, dtmf_cmd, ";
                $query .= "color, idm_relay_type, idm_relay_host, idm_relay_port, idm_relay_username, idm_relay_password, idm_relay_relay ";
                $query .= "FROM commands WHERE id = ?";
                if(!DBUtils::executeSql('cfg', 'camera', $query, $sqlData)) {
                    return array (
                        "name" => "Duplicate",
                        "id" => "duplicate",
                        "data" => $query,
                        "msg" => '<div id="error-msg">' . GuiUtils::getSessionLocalizedString('dbLoadFail') . '</div>',
                        "success" => false);
                }
                break;
            case "camera":
                $sqlData = array('types' => 'i',
                    'id' => $origId);

                $query = "INSERT INTO cameras (name, camera_type_id, url,hostname, port, username, password, realm) ";
                $query .= "SELECT CONCAT(name, ' Copy " . date("Y-m-d H:i:s") . "') as name, camera_type_id, url,hostname, port, username, password, realm ";
                $query .= "FROM cameras WHERE id = ?";
                if(!DBUtils::executeSql('cfg', 'camera', $query, $sqlData)) {
                    return array (
                        "name" => "Duplicate",
                        "id" => "duplicate",
                        "data" => $query,
                        "msg" => '<div id="error-msg">' . GuiUtils::getSessionLocalizedString('dbLoadFail') . '</div>',
                        "success"=>false);
                }
                break;
            case "sipDefault":
                $sqlData = array('types' => 'i',
                    'id' => $origId);

                $query = "INSERT INTO sip_defaults (name, SIPServerURL, SIPAllowMakeCall, SIPNotificationBasedSystem, ";
                $query .= "SIPDoorSystemType, ControlsPassword, SIPServerPort, SIPUseOnlyUdp, MainDoorMSN, CallButtonName, ";
                $query .= "OpenDoorDTMF, AnswerDoorDTMF, HangUpDTMF, DelayDTMF, InbandDTMF, InbandDTMFClockRate, ";
                $query .= "InbandDTMFChannels, InbandDTMFSamplesPerFrame, InbandDTMFBitsPerSample, SIPUseStunServers, STUNServer1, ";
                $query .= "STUNServer2,STUNServer3, STUNServer4, SIPDebugMode) ";
                $query .= "SELECT CONCAT(name, ' Copy " . date("Y-m-d H:i:s") . "') as name, ";
                $query .= "SIPServerURL, SIPAllowMakeCall, SIPNotificationBasedSystem, ";
                $query .= "SIPDoorSystemType, ControlsPassword, SIPServerPort, SIPUseOnlyUdp, MainDoorMSN, CallButtonName, ";
                $query .= "OpenDoorDTMF, AnswerDoorDTMF, HangUpDTMF, DelayDTMF, InbandDTMF, InbandDTMFClockRate, ";
                $query .= "InbandDTMFChannels, InbandDTMFSamplesPerFrame, InbandDTMFBitsPerSample, SIPUseStunServers, STUNServer1, ";
                $query .= "STUNServer2,STUNServer3, STUNServer4, SIPDebugMode ";
                $query .= "FROM sip_defaults WHERE id = ?";
                if(!DBUtils::executeSql('cfg', 'sipDefault', $query, $sqlData)) {
                    return array (
                        "name" => "Duplicate",
                        "id" => "duplicate",
                        "data" => $query,
                        'msg' => '<div id="error-msg">' . GuiUtils::getSessionLocalizedString('dbLoadFail') . '</div>',
                        'success'=>false);
                }
                break;
            default:
                return array (
                    "name" => "Duplicate",
                    "id" => "duplicate",
                    "data" => $data,
                    "msg" => "<div id=\"error-msg\">" . GuiUtils::getSessionLocalizedString('dbLoadFail') . "</div>",
                    "success"=>false);
        }

        return array (
           "name" => "Duplicate",
           "id" => "duplicate",
           "data" => $data,
           "msg" => "<div id=\"load-ok-msg\">" . GuiUtils::getSessionLocalizedString('dbLoadOk') . "</div>",
           "commandType" => isset($data['commandType']) ? $data['commandType'] : 'NoNe',
           "success"=>true);
    }

    public static function captureImage($camUrl, $filePath){
        $con = fopen($camUrl,"r") ;
        if($con)
        {
            $i = 0;
            $maxIteration = 10241; // 5Mb
            $data = "";
            while (substr_count($data, "Content-Length") != 2 && $i < $maxIteration){
                $data .= fread($con, 512);
                $i++;
            }
            if($i < $maxIteration) {
                $start = strpos($data, "\xff");
                $end = strpos($data, "\n--", $start)-1;
                $frame = substr($data, $start, $end-$start);
                file_put_contents($filePath, $frame);
            } else {
                return 0;
            }
        } else {
             return 0;
        }
        fclose($con);

        return 1;
    }

    public static function captureDoorImages() {
        global $videoMemCfg;
        $callerId = isset($_GET['callerId']) ? $_GET['callerId'] : 0;
        $destId = isset($_GET['destId']) ? $_GET['destId'] : 0;

        if($callerId != 0 && $destId != 0) {
            $query = "SELECT apartment_id FROM sip_accounts WHERE username=?";
            $sqlData = array('types' => 's',
                'username' => $destId
            );

            $apartmentData = DBUtils::executeSql('sip', 'captureImage', $query, $sqlData, false, true);
            if(isset($apartmentData) && isset($apartmentData[0]['apartment_id'])) {
                $imageDir = $videoMemCfg['imagesHome'] . '/' . $apartmentData[0]['apartment_id'];
                if(file_exists($imageDir) || mkdir($imageDir)){
                    $date = new \DateTime();
                    $captureTime = $date->getTimestamp();
                    $folderTime = $captureTime - $captureTime % 100;
                    $imageDir .= "/" . $folderTime;
                    if(file_exists($imageDir)){
                        return array (
                            "name" => "Capture",
                            "id" => "Capture",
                            "a" => $imageDir,
                            "msg" => "<div id=\"error-msg\">" . GuiUtils::getSessionLocalizedString('captureFail') . "</div>",
                            "success"=>false);
                    }

                    $imageDir .= "/" . $callerId;
                    if(mkdir($imageDir, 0777, true)){
                        $query = "select c.* from cameras c, doors d, door_cam_link dc where d.msn=? ";
                        $query .= "and dc.door_id=d.id and c.id = dc.cam_id;";
                        $sqlData = array('types' => 's',
                            'msn' => $callerId
                        );

                        $camData = DBUtils::executeSql('cfg', 'captureImage', $query, $sqlData, false, true);
                        $captureCount = 0;
                        foreach($camData as $cam) {
                            $camUrl = $cam['url'];
                            if(isset($camUrl) && strlen($camUrl) > 0){
                                if(isset($cam['username']) && strlen($cam['username']) > 0){
                                    $camUrl = str_replace("http://", 'http://' . $cam['username'] . ':' . $cam['password'] . '@', $cam['url']);
                                }
                                $filePath = $imageDir . "/" . $captureTime . "-" . $cam['id'] . ".jpg";
                                $captureCount += ApiSupport::captureImage($camUrl, $filePath);
                            }
                        }

                        return array (
                            "name" => "Capture",
                            "id" => "Capture",
                            "msg" => "<div id=\"error-msg\">" . GuiUtils::getSessionLocalizedString('captureOk') . "</div>",
                            "success"=>true);
                    }
                }
            }
        }

        return array (
            "name" => "Capture",
            "id" => "Capture",
            "a" => $destId,
            "msg" => "<div id=\"error-msg\">" . GuiUtils::getSessionLocalizedString('captureFail') . "</div>",
            "success"=>false);
    }

    public static function scandir_folders($d) {
       return array_filter(scandir($d), function ($f) use($d) {
           return is_dir($d . DIRECTORY_SEPARATOR . $f);
       });
    }

    public static function lastCalls() {
        global $videoMemCfg;

        if(!isset($_GET['user']) || !isset($_GET['key'])){
            return array(
                "id" => "getLastCalls",
                "status" => 1,
                "newCalls" => array(),
                "error" => "Missing information!");
        }

        $lastUpdate = isset($_GET['lastCall']) ? $_GET['lastCall'] : 1410104922;

        $sql = 'select id, apartment_id, username, password, authname from sip_accounts where username=';
        $apartmentData = DBUtils::getDbData('sip', 'get config', $sql, $_GET['user'], 1);
        if(empty($apartmentData) || isset($apartmentData['error'])) {
            return array(
                "id" => "getLastCalls",
                "status" => 2,
                "lastCall" => $lastUpdate,
                "newCalls" => array(),
                "error" => "No information for you!");
        }
        $localKey = "1D0rG3tUs3r:" . $apartmentData[0]['password'];
        $localKey = md5($localKey);

        if($localKey != $_GET['key']) {
            return array(
                "id" => "getLastCalls",
                "status" => 3,
                "lastCall" => $lastUpdate,
                "newCalls" => array(),
                "error" => "No information for you!");
        }

        $imageDir = $videoMemCfg['imagesHome'] . '/' . $apartmentData[0]['apartment_id'];

        $newCalls = array_filter(
            array_diff(scandir($imageDir), array('..', '.')),
            function ($value) use($lastUpdate) {
                return ($value > $lastUpdate);
            }
        );
        rsort($newCalls);
        if(is_array($newCalls) && count($newCalls) > 0) {
            $callsDetails = array();
            foreach ($newCalls as $call) {
                $callDir = $imageDir . "/" . $call;
                $doorsInCall = array_diff(scandir($callDir), array('..', '.'));
                foreach($doorsInCall as $door) {
                    $camsDir = $callDir . "/" . $door;
                    $camsImgFiles = array_diff(scandir($camsDir), array('..', '.'));
                    $camsImages = array();
                    foreach($camsImgFiles as $imgFile){
                        $imgTime = strstr($imgFile, '-', true);
                        $camId = str_replace("-", "", strstr(strstr($imgFile, '-'), ".", true));

                        $sql = 'select id, name from cameras where id=';
                        $camData = DBUtils::getDbData('cfg', 'get config', $sql, $camId, 1);
                        $camName = "Unknown Camera!";
                        if(!empty($camData) && !isset($camData['error'])) {
                            $camName = $camData[0]['name'];
                        }
                        $camsImages[] = array (
                            "cameraId" => $camId,
                            "cameraName" => $camName,
                            "imageTime" => date('Y-m-d H:i:s'),
                            "imageName" => $imgFile
                        );
                    }

                    $sql = 'select id, name from doors where msn=';
                    $doorData = DBUtils::getDbData('cfg', 'get config', $sql, $door, 1);
                    $doorName = "Unknown Door!";
                    if(!empty($doorData) && !isset($doorData['error'])) {
                            $doorName = $doorData[0]['name'];
                    }

                    $callsDetails[] = array(
                        "callId" => $call,
                        "doorId" => $door,
                        "doorName" => $doorName,
                        "doorUrlChunk" => $videoMemCfg['imagesUrlBase'] . "/" . $apartmentData[0]['apartment_id'] . "/" . $call . "/" . $door,
                        "doorCamImages" => $camsImages
                    );
                }
            }
            return array(
                "id" => "getLastCalls",
                "status" => 0,
                "lastCall" => $lastUpdate,
                "newCalls" => $callsDetails);
            ;
        }

        return array(
            "id" => "getLastCalls",
            "status" => 0,
            "newCalls" => array());
    }
}
